@@ -1,5 +1,5 @@
 # ------------------------------------------------------------------------
-# Copyright 2020-2021, Harald Lieder, mailto:harald.lieder@outlook.com
+# Copyright 2020-2022, Harald Lieder, mailto:harald.lieder@outlook.com
 # License: GNU AFFERO GPL 3.0, https://www.gnu.org/licenses/agpl-3.0.html
 #
 # Part of "PyMuPDF", a Python binding for "MuPDF" (http://mupdf.com), a
@@ -7,6 +7,7 @@
 # maintained and developed by Artifex Software, Inc. https://artifex.com.
 # ------------------------------------------------------------------------
 import sys
+
 from fitz.fitz import *
 
 # define the supported colorspaces for convenience
@@ -17,10 +18,31 @@ csRGB = fitz.csRGB
 csGRAY = fitz.csGRAY
 csCMYK = fitz.csCMYK
 
-# create the TOOLS object
+# create the TOOLS object.
+#
+# Unfortunately it seems that this is never be destructed even if we use an
+# atexit() handler, which makes MuPDF's Memento list it as a leak. In fitz.i
+# we use Memento_startLeaking()/Memento_stopLeaking() when allocating
+# the Tools instance so at least the leak is marked as known.
+#
 TOOLS = fitz.Tools()
+TOOLS.thisown = True
 fitz.TOOLS = TOOLS
 
+# This atexit handler runs, but doesn't cause ~Tools() to be run.
+#
+import atexit
+
+
+def cleanup_tools(TOOLS):
+    # print(f'cleanup_tools: TOOLS={TOOLS} id(TOOLS)={id(TOOLS)}')
+    # print(f'TOOLS.thisown={TOOLS.thisown}')
+    del TOOLS
+    del fitz.TOOLS
+
+
+atexit.register(cleanup_tools, TOOLS)
+
 if fitz.VersionFitz != fitz.TOOLS.mupdf_version():
     v1 = fitz.VersionFitz.split(".")
     v2 = fitz.TOOLS.mupdf_version().split(".")
@@ -33,7 +55,6 @@ if fitz.VersionFitz != fitz.TOOLS.mupdf_version():
 # copy functions in 'utils' to their respective fitz classes
 import fitz.utils
 
-
 # ------------------------------------------------------------------------------
 # General
 # ------------------------------------------------------------------------------
@@ -71,6 +92,7 @@ fitz.Document.tobytes = fitz.Document.write
 fitz.Document.subset_fonts = fitz.utils.subset_fonts
 fitz.Document.get_oc = fitz.utils.get_oc
 fitz.Document.set_oc = fitz.utils.set_oc
+fitz.Document.xref_copy = fitz.utils.xref_copy
 
 
 # ------------------------------------------------------------------------------
@@ -109,6 +131,8 @@ fitz.Page.write_text = fitz.utils.write_text
 fitz.Page.get_label = fitz.utils.get_label
 fitz.Page.get_image_rects = fitz.utils.get_image_rects
 fitz.Page.get_textpage_ocr = fitz.utils.get_textpage_ocr
+fitz.Page.delete_image = fitz.utils.delete_image
+fitz.Page.replace_image = fitz.utils.replace_image
 
 # ------------------------------------------------------------------------
 # Annot
@@ -160,7 +184,7 @@ def restore_aliases():
         if callable(fname):
 
             def deprecated_function(*args, **kw):
-                msg = "'%s' removed from %s after v1.19.0 - use '%s'." % (
+                msg = "'%s' removed from %s after v1.19 - use '%s'." % (
                     old,
                     objname,
                     new,
@@ -183,9 +207,7 @@ def restore_aliases():
         try:
             if callable(fname) or type(fname) is property:
                 eigen.__doc__ = (
-                    "*** Deprecated and removed in version following 1.19.0 - use '%s'. ***\n"
-                    % new
-                    + x
+                    "*** Deprecated and removed after v1.19 - use '%s'. ***\n" % new + x
                 )
         except:
             pass
@@ -425,7 +447,7 @@ def restore_aliases():
     _alias(fitz, "PaperSize", "paper_size")
     _alias(fitz, "PaperRect", "paper_rect")
     _alias(fitz, "paperSizes", "paper_sizes")
-    _alias(fitz, "ImageProperties", "image_properties")
+    _alias(fitz, "ImageProperties", "image_profile")
     _alias(fitz, "planishLine", "planish_line")
     _alias(fitz, "getTextLength", "get_text_length")
     _alias(fitz, "getTextlength", "get_text_length")
@@ -442,31 +464,15 @@ Built for Python %i.%i on %s (%i-bit).
     sys.version_info[0],
     sys.version_info[1],
     sys.platform,
-    64 if sys.maxsize > 2 ** 32 else 32,
+    64 if sys.maxsize > 2**32 else 32,
 )
 
-restore_aliases()
+if VersionBind.startswith("1.19"):  # don't generate aliases after v1.19.*
+    restore_aliases()
 
-# some special geometry objects
-def EMPTY_RECT():
-    return Rect(FZ_MAX_INF_RECT, FZ_MAX_INF_RECT, FZ_MIN_INF_RECT, FZ_MIN_INF_RECT)
-
-
-def INFINITE_RECT():
-    return Rect(FZ_MIN_INF_RECT, FZ_MIN_INF_RECT, FZ_MAX_INF_RECT, FZ_MAX_INF_RECT)
-
-
-def EMPTY_IRECT():
-    return IRect(FZ_MAX_INF_RECT, FZ_MAX_INF_RECT, FZ_MIN_INF_RECT, FZ_MIN_INF_RECT)
-
-
-def INFINITE_IRECT():
-    return IRect(FZ_MIN_INF_RECT, FZ_MIN_INF_RECT, FZ_MAX_INF_RECT, FZ_MAX_INF_RECT)
-
-
-def INFINITE_QUAD():
-    return INFINITE_RECT().quad
-
-
-def EMPTY_QUAD():
-    return EMPTY_RECT().quad
+pdfcolor = dict(
+    [
+        (k, (r / 255, g / 255, b / 255))
+        for k, (r, g, b) in fitz.utils.getColorInfoDict().items()
+    ]
+)
